﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;

namespace SyumasGen
{
    class ImageGenerator : IDisposable
    {
        private Bitmap _bmp = null;
        private Graphics _g = null;

        private const int _BaseWidth = 512;
        private const int _BaseHeight = 384;

        /// <summary>
        /// インスタンスを初期化します。
        /// </summary>
        public ImageGenerator()
        {
            // グラフィックス描画の準備
            _bmp = new Bitmap(_BaseWidth, _BaseHeight);
            _g = Graphics.FromImage(_bmp);

            // レンダリング品質にアンチエイリアスを指定
            _g.SmoothingMode = SmoothingMode.AntiAlias;
            _g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
        }

        /// <summary>
        /// 割り当てられているリソースを破棄します。
        /// </summary>
        public void Dispose()
        {
            try
            {
                _g.Dispose();
                _bmp.Dispose();
            }
            catch (Exception)
            { }
        }

        /// <summary>
        /// フレーム画像を生成します。
        /// </summary>
        /// <param name="rank"></param>
        /// <param name="option"></param>
        /// <param name="saveFilePath"></param>
        public void DrawFrame(RankingData rank, Settings option, string saveFilePath)
        {
            // 描画エリアのクリア
            _g.Clear(Color.FromArgb(0, Color.Black));

            // フレーム情報
            FrameInfoData frameData = option.FrameInfoList[rank.Mark];

            // フレーム画像描画
            string framePath = Path.Combine(Application.StartupPath, frameData.FrameImageFile);
            _g.DrawImage(Image.FromFile(framePath), option.FramePos);

            // 順位変動描画
            TextInfoData rankStatInfo = option.TextInfoList[TextInfoKind.RankStat];
            Point rankStatPos = new Point(rankStatInfo.PosX, rankStatInfo.PosY);
            Font rankStatFont = new Font(rankStatInfo.FontName, rankStatInfo.FontSize, FontStyle.Bold, GraphicsUnit.Pixel);

            DrawFuchidoriText(rank.RankStat, rankStatFont, rankStatPos, frameData.RankColor, Color.White, 0.5f);

            rankStatFont.Dispose();

            // 順位
            TextInfoData rankInfo = option.TextInfoList[TextInfoKind.Rank];
            Point rankPos = new Point(rankInfo.PosX, rankInfo.PosY);
            Point rankShadowPos = rankPos + new Size(3, 3);
            Font rankFont = new Font(rankInfo.FontName, rankInfo.FontSize, FontStyle.Bold | FontStyle.Italic, GraphicsUnit.Pixel);
            Color shadowColor = Color.FromArgb(128, Color.Black);

            DrawFuchidoriText(rank.Rank.ToString(), rankFont, rankShadowPos, shadowColor, shadowColor, 1.5f);
            DrawFuchidoriText(rank.Rank.ToString(), rankFont, rankPos, frameData.RankColor, Color.White, 1.5f);

            int rankWidth = (int)_g.MeasureString(rank.Rank.ToString(), rankFont).Width;
            rankFont.Dispose();

            // タイトル
            TextInfoData titleInfo = option.TextInfoList[TextInfoKind.Title];
            Font titleFont = new Font(titleInfo.FontName, titleInfo.FontSize, FontStyle.Bold);
            int posX = rankInfo.PosX + rankWidth;

            _g.DrawString(rank.Title, titleFont, Brushes.Black, posX, titleInfo.PosY);

            titleFont.Dispose();

            // ポイント
            TextInfoData videoDatainfo = option.TextInfoList[TextInfoKind.VideoData];
            Font videoDataFont = new Font(videoDatainfo.FontName, videoDatainfo.FontSize, FontStyle.Bold);
            string pts = rank.Pts.ToString();
            Point pos = new Point(5, videoDatainfo.PosY);

            _g.DrawString(pts, videoDataFont, Brushes.Black, pos);
            pos.X += GetStringWidth(pts, videoDataFont, pos);
            _g.DrawString("pts", videoDataFont, Brushes.Black, pos);
            pos.X += GetStringWidth("pts", videoDataFont, pos);

            // 登録
            string mylist = rank.Mylist.ToString();
            pos.X += 5;

            _g.DrawString("登録:", videoDataFont, Brushes.Black, pos);
            pos.X += GetStringWidth("登録:", videoDataFont, pos);
            _g.DrawString(mylist, videoDataFont, Brushes.Black, pos);
            pos.X += GetStringWidth(mylist, videoDataFont, pos);

            // 再生
            string play = rank.Play.ToString();
            pos.X += 5;

            _g.DrawString("再生:", videoDataFont, Brushes.Black, pos);
            pos.X += GetStringWidth("再生:", videoDataFont, pos);
            _g.DrawString(play, videoDataFont, Brushes.Black, pos);
            pos.X += GetStringWidth(play, videoDataFont, pos);

            // 投稿日
            string date = rank.Date.ToString("[MM/dd]");
            pos = new Point(_BaseWidth, videoDatainfo.PosY);
            pos.X -= GetStringWidth(date, videoDataFont, pos) + 10;

            _g.DrawString(date, videoDataFont, Brushes.Black, pos);

            // 動画ID
            string videoId = rank.VideoId;
            pos.X -= GetStringWidth(videoId, videoDataFont, pos) + 5;

            _g.DrawString(videoId, videoDataFont, Brushes.Black, pos);

            // 作者名
            string author = rank.Author;
            pos.X -= GetStringWidth(author, videoDataFont, pos) + 5;

            _g.DrawString(author, videoDataFont, Brushes.Black, pos);

            videoDataFont.Dispose();

            // 補足
            if (rank.Remarks != "")
            {
                TextInfoData remarksInfo = option.TextInfoList[TextInfoKind.Remarks];
                Font remarksFont = new Font(remarksInfo.FontName, remarksInfo.FontSize, FontStyle.Bold, GraphicsUnit.Pixel);
                pos.X = _BaseWidth - 10;
                pos.Y = remarksInfo.PosY;
                int remarksWidth = GetStringWidth(rank.Remarks, remarksFont, pos);
                pos.X -= remarksWidth;

                DrawFuchidoriText(rank.Remarks, remarksFont, pos, Color.Black, Color.White, 5.0f);
                _g.DrawString(rank.Remarks, remarksFont, Brushes.Black, pos);

                remarksFont.Dispose();
            }

            // 保存
            _bmp.Save(saveFilePath, System.Drawing.Imaging.ImageFormat.Png);
        }

        /// <summary>
        /// 順位変動情報画像を生成します。
        /// </summary>
        /// <param name="rank"></param>
        /// <param name="option"></param>
        /// <param name="saveFilePath"></param>
        public void DrawRankInfoText(RankingData rank, Settings option, string saveFilePath)
        {
            // 描画エリアのクリア
            _g.Clear(Color.FromArgb(0, Color.Black));

            // 表示内容
            string text = rank.DiffRank.ToString(option.FrameInfoList[rank.Mark].Text);

            // フォントの指定
            TextInfoData info = option.TextInfoList[TextInfoKind.RankInfo];
            Font f = new Font(info.FontName, info.FontSize, FontStyle.Bold, GraphicsUnit.Pixel);

            // 文字列描画
            GraphicsPath path = new GraphicsPath();

            path.AddString(
                text,
                f.FontFamily,
                (int)f.Style,
                f.Size,
                new Point(0, 0),
                StringFormat.GenericDefault);

            Color c = option.FrameInfoList[rank.Mark].RankColor;

            _g.FillPath(new SolidBrush(c), path);
            _g.DrawPath(new Pen(Color.White, 1.0f), path);

            // 保存
            Size size = _g.MeasureString(text, f).ToSize();
            Image saveImg = _bmp.Clone(new Rectangle(0, 0, size.Width, size.Height), _bmp.PixelFormat);

            string savePath = Path.Combine(Application.StartupPath, @"rank\text" + rank.Rank.ToString() + ".png");
            saveImg.Save(saveFilePath, System.Drawing.Imaging.ImageFormat.Png);

            return;
        }

        /// <summary>
        /// 縁取り文字の描画を行います。
        /// </summary>
        /// <param name="s"></param>
        /// <param name="f"></param>
        /// <param name="pos"></param>
        /// <param name="textColor"></param>
        /// <param name="fuchiColor"></param>
        /// <param name="thickness"></param>
        private void DrawFuchidoriText(string s, Font f, Point pos, Color textColor, Color fuchiColor, float thickness)
        {
            GraphicsPath path = new GraphicsPath();
            path.AddString(s, f.FontFamily, (int)f.Style, f.Size, pos, StringFormat.GenericDefault);
            _g.FillPath(new SolidBrush(textColor), path);
            _g.DrawPath(new Pen(new SolidBrush(fuchiColor), thickness), path);
        }

        /// <summary>
        /// 指定した文字列を指定したフォントで描画したときの横幅を取得します。
        /// </summary>
        /// <param name="s"></param>
        /// <param name="f"></param>
        /// <param name="pos"></param>
        /// <returns></returns>
        private int GetStringWidth(string s, Font f, Point pos)
        {
            SizeF strSize = _g.MeasureString(s, f);
            RectangleF strRect = new RectangleF(pos, strSize);
            CharacterRange[] cr = { new CharacterRange(0, s.Length) };

            StringFormat sf = new StringFormat();
            sf.SetMeasurableCharacterRanges(cr);

            Region[] r = _g.MeasureCharacterRanges(s, f, strRect, sf);
            float width = r[0].GetBounds(_g).Width;
            return (int)(width);
        }
    }
}
