﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Text;

namespace SyumasGen
{
    /// <summary>
    /// 設定に関するクラス
    /// </summary>
    class Settings
    {
        /// <summary>フレームの表示位置を取得します。</summary>
        public Point FramePos { get; private set; }

        /// <summary>フレーム情報の設定を取得します。</summary>
        public Dictionary<string, FrameInfoData> FrameInfoList { get; private set; }

        /// <summary>順位情報の設定を取得します。</summary>
        public Dictionary<TextInfoKind, TextInfoData> TextInfoList { get; private set; }

        /// <summary>
        /// 設定ファイルの文字データを読み取ります。
        /// </summary>
        /// <param name="lpAppName"></param>
        /// <param name="lpKeyName"></param>
        /// <param name="lpDefault"></param>
        /// <param name="lpReturnedString"></param>
        /// <param name="nSize"></param>
        /// <param name="lpFileName"></param>
        /// <returns></returns>
        [DllImport("kernel32.dll")]
        private static extern uint GetPrivateProfileString(
            string lpAppName,
            string lpKeyName,
            string lpDefault,
            StringBuilder lpReturnedString,
            uint nSize,
            string lpFileName);

        /// <summary>
        /// 設定ファイルの数値データを読み取ります。
        /// </summary>
        /// <param name="lpAppName"></param>
        /// <param name="lpKeyName"></param>
        /// <param name="nDefault"></param>
        /// <param name="lpFileName"></param>
        /// <returns></returns>
        [DllImport("kernel32.dll")]
        private static extern uint GetPrivateProfileInt(
            string lpAppName,
            string lpKeyName,
            int nDefault,
            string lpFileName);

        /// <summary>
        /// 設定ファイルを読み込みます。
        /// </summary>
        /// <param name="iniPath"></param>
        public void ReadIniFile(string iniPath)
        {
            try
            {
                // フレーム表示座標
                int x = (int)GetPrivateProfileInt("FramePos", "X", 0, iniPath);
                int y = (int)GetPrivateProfileInt("FramePos", "Y", 0, iniPath);
                FramePos = new Point(x, y);

                // フレーム情報取得
                FrameInfoList = new Dictionary<string, FrameInfoData>();

                int num = (int)GetPrivateProfileInt("FrameNum", "FrameNum", 0, iniPath);

                for (int i = 1; i <= num; i++)
                {
                    AddFrameInfoList(iniPath, "FrameInfo" + i.ToString());
                }

                // テキスト情報取得
                TextInfoList = new Dictionary<TextInfoKind, TextInfoData>();

                AddTextInfoList(iniPath, "RankInfoText", TextInfoKind.RankInfo);
                AddTextInfoList(iniPath, "TextRankStatus", TextInfoKind.RankStat);
                AddTextInfoList(iniPath, "TextRank", TextInfoKind.Rank);
                AddTextInfoList(iniPath, "TextTitle", TextInfoKind.Title);
                AddTextInfoList(iniPath, "TextVideoData", TextInfoKind.VideoData);
                AddTextInfoList(iniPath, "TextRemarks", TextInfoKind.Remarks);
            }
            catch (Exception ex)
            {
                throw ex;   // とりあえず
            }
        }

        /// <summary>
        /// フレームに関する設定を読み込みます。
        /// </summary>
        /// <param name="iniPath"></param>
        /// <param name="entry"></param>
        private void AddFrameInfoList(string iniPath, string entry)
        {
            FrameInfoData data = new FrameInfoData();
            StringBuilder sb = new StringBuilder(64);

            // 順位変動識別文字
            GetPrivateProfileString(entry, "Key", "", sb, (uint)sb.Capacity, iniPath);
            string key = sb.ToString();

            // フレーム画像ファイル
            GetPrivateProfileString(entry, "FrameImageFile", "", sb, (uint)sb.Capacity, iniPath);
            data.FrameImageFile = sb.ToString();

            // 色
            GetPrivateProfileString(entry, "Color", "", sb, (uint)sb.Capacity, iniPath);
            //string[] c = sb.ToString().Split(',');
            //data.RankColor = Color.FromArgb(Int32.Parse(c[0]), Int32.Parse(c[1]), Int32.Parse(c[2]));
            data.RankColor = Color.FromName(sb.ToString());

            // 順位変動情報文字列
            GetPrivateProfileString(entry, "Text", "", sb, (uint)sb.Capacity, iniPath);
            data.Text = sb.ToString();

            // リストに追加
            FrameInfoList.Add(key, data);
        }

        /// <summary>
        /// テキストに関する設定を読み込みます。
        /// </summary>
        /// <param name="iniPath"></param>
        /// <param name="entry"></param>
        /// <param name="kind"></param>
        private void AddTextInfoList(string iniPath, string entry, TextInfoKind kind)
        {
            TextInfoData data = new TextInfoData();
            StringBuilder sb = new StringBuilder(64);

            // フォント
            GetPrivateProfileString(entry, "Font", "", sb, (uint)sb.Capacity, iniPath);
            data.FontName = sb.ToString();

            // フォントサイズ
            data.FontSize = (float)GetPrivateProfileInt(entry, "FontSize", 0, iniPath);

            // 座標
            data.PosX = (int)GetPrivateProfileInt(entry, "PosX", 0, iniPath);
            data.PosY = (int)GetPrivateProfileInt(entry, "PosY", 0, iniPath);

            TextInfoList.Add(kind, data);
        }
    }

    /// <summary>
    /// フレーム情報
    /// </summary>
    public struct FrameInfoData
    {
        /// <summary>フレーム画像ファイル</summary>
        public string FrameImageFile { get; set; }

        /// <summary>ランクカラー</summary>
        public Color RankColor { get; set; }

        /// <summary>フレーム表示前順位変動情報の文字列</summary>
        public string Text { get; set; }
    }

    /// <summary>
    /// ランキング情報設定項目
    /// </summary>
    public enum TextInfoKind
    {
        /// <summary>フレーム表示前順位変動情報（"8 Rank Up!"等）</summary>
        RankInfo,

        /// <summary>フレーム順位変動情報（"新","9(▲8)"等）</summary>
        RankStat,

        /// <summary>順位</summary>
        Rank,

        /// <summary>タイトル</summary>
        Title,

        /// <summary>動画情報</summary>
        VideoData,

        /// <summary>補足</summary>
        Remarks,
    }

    /// <summary>
    /// ランキング情報設定
    /// </summary>
    public struct TextInfoData
    {
        /// <summary>フォント名</summary>
        public string FontName { get; set; }

        /// <summary>フォントサイズ</summary>
        public float FontSize { get; set; }

        /// <summary>表示横座標</summary>
        public int PosX { get; set; }

        /// <summary>表示縦座標</summary>
        public int PosY { get; set; }
    }
}
